'use strict';

var web = require('solid-js/web');
var solidJs = require('solid-js');
var store = require('solid-js/store');
var solidjsStorex = require('solidjs-storex');

// src/WindowManager.tsx

// src/eventPathPolyfill.ts
function composePath(el) {
  var path = [];
  while (el) {
    path.push(el);
    if (el.tagName === "HTML") {
      path.push(document);
      path.push(window);
      return path;
    }
    el = el.parentElement;
  }
}
function eventPath(event) {
  return event.path || event.composedPath && event.composedPath() || composePath(event.target);
}
var windowPreferencesStore_default = solidjsStorex.defineStore({
  state: {},
  options: {
    persistent: true,
    storageKey: "windows-preferences-state",
    storageThrottle: 250
  },
  actions: (state, set) => ({
    save: (key, props) => {
      set(key, props);
    },
    get: (key) => {
      if (state.hasOwnProperty(key)) {
        return state[key];
      }
    }
  })
});
function findWindowPreferences(state, key) {
  if (state.hasOwnProperty(key)) {
    return state[key];
  }
}

// src/store/windowStore.ts
var [windowPreferencesState, { save: saveWindowPreferences }] = windowPreferencesStore_default();
exports.windowStore = solidjsStorex.defineStore({
  state: {
    windows: {}
  },
  options: {
    persistent: true,
    storageKey: "windows-state",
    storageThrottle: 250
  },
  actions: (state, set) => ({
    openWindow: (component, props) => {
      Object.keys(state?.windows || {}).forEach((item) => {
        if (state.windows[item].attrs.minimized == false) {
          set("windows", item, "attrs", "zIndex", (i) => ++i);
        }
      });
      let key = generateWindowKey({ component, props });
      set("windows", key, {
        component,
        props,
        attrs: { ...getDefaultWindowsAttrs({ component, props }), key }
      });
    },
    closeWindow: (key) => {
      set(
        store.produce((state2) => {
          if (state2?.windows?.hasOwnProperty(key)) {
            delete state2.windows[key];
          }
        })
      );
    },
    moveWindow: (key, pos) => {
      set("windows", key, "attrs", "pos", pos);
      saveWindowPreferences(state.windows[key].component, (attrs) => ({ ...attrs, pos: state.windows[key].attrs.pos }));
    },
    resizeWindow: (key, size) => {
      const minWidth = 300;
      const minHeight = 300;
      set("windows", key, "attrs", "size", (currentSize) => {
        let newSize = [...size(currentSize)];
        return [
          newSize[0] < minWidth ? minWidth : newSize[0],
          newSize[1] < minHeight ? minHeight : newSize[1]
        ];
      });
      saveWindowPreferences(state.windows[key].component, (attrs) => ({
        ...attrs,
        size: state.windows[key].attrs.size
      }));
    },
    focusWindow: (key) => {
      Object.keys(state?.windows || {}).forEach((item) => {
        if (state.windows[item].attrs.minimized == false) {
          set("windows", item, "attrs", "zIndex", (i) => ++i);
        }
      });
      set("windows", key, "attrs", "minimized", false);
      set("windows", key, "attrs", "zIndex", 1);
    },
    updateWindowState: (key, newState) => {
      set("windows", key, "attrs", "state", (s) => {
        return { ...s, ...newState };
      });
    },
    maximizeWindow: (key, maximized) => {
      set("windows", key, "attrs", "maximized", maximized);
    },
    minimizeWindow: (key, minimized) => {
      set("windows", key, "attrs", "minimized", minimized);
      if (minimized == true) {
        set("windows", key, "attrs", "zIndex", 0);
        Object.keys(state?.windows || {}).forEach((item) => {
          if (state.windows[item].attrs.minimized == false) {
            set("windows", item, "attrs", "zIndex", (i) => i > 1 ? --i : i);
          }
        });
      }
    }
  })
});
var getDefaultWindowsAttrs = (props) => {
  const defaultAttrs = {
    maximized: false,
    minimized: false,
    // TODO: calculate center of window minus half window size
    // pos: [window.innerWidth / 2 || 200, window.innerHeight / 2 || 200],
    pos: [200, 200],
    size: [500, 400],
    zIndex: 1,
    icon: null,
    key: "",
    state: { title: "", loading: true }
  };
  if (props.component) {
    let preferences = findWindowPreferences(windowPreferencesState, props.component);
    if (preferences) {
      if (preferences.pos)
        defaultAttrs.pos = preferences.pos;
      if (preferences.size)
        defaultAttrs.size = preferences.size;
    }
  }
  return defaultAttrs;
};
var generateWindowKey = (props) => {
  return new Array(15).join().replace(/(.|$)/g, function() {
    return (Math.random() * 36 | 0).toString(36)[Math.random() < 0.5 ? "toString" : "toUpperCase"]();
  });
};
var _tmpl$ = /* @__PURE__ */ web.template(`<style>
        .window-manager-icon-empty {
          display: flex;
          flex-direction: row;
          align-items: center;
          justify-content: center;
          font-size: 11px;
          color: #5c5c5c;
          font-family: sans-serif;
          text-transform: uppercase;
          background: #ccc;
          border-radius: 100%;
          width: 18px;
          height: 18px;
          margin: 10px;
          cursor: inherit;
        }
      `);
var _tmpl$2 = /* @__PURE__ */ web.template(`<span class="window-manager-icon-empty">`);
function WindowIcon(attrs) {
  const [props] = solidJs.splitProps(attrs, ["icon", "name"]);
  return [web.createComponent(solidJs.Show, {
    get when() {
      return props.icon;
    },
    get fallback() {
      return (() => {
        const _el$2 = _tmpl$2();
        web.insert(_el$2, () => String(props?.name || " ").charAt(0));
        return _el$2;
      })();
    },
    get children() {
      return web.createComponent(solidJs.ErrorBoundary, {
        fallback: (error) => [],
        get children() {
          return web.createComponent(solidJs.Suspense, {
            get fallback() {
              return [];
            },
            get children() {
              return web.createComponent(web.Dynamic, {
                get component() {
                  return props.icon;
                }
              });
            }
          });
        }
      });
    }
  }), _tmpl$()];
}

// src/Taskbar.tsx
var _tmpl$3 = /* @__PURE__ */ web.template(`<div class="window-manager-taskbar-placeholder">`);
var _tmpl$22 = /* @__PURE__ */ web.template(`<div class="window-manager-taskbar"><div class="window-manager-taskbar-wrapper">`);
var _tmpl$32 = /* @__PURE__ */ web.template(`<style>
        .window-manager-taskbar-placeholder {
          width: 100%;
          height: 50px;
        }
        .window-manager-taskbar {
          position: fixed;
          top: 0;
          left: 0;
          right: 0;
          width: 100%;
          height: 50px;
          z-index: 100;
          background: rgba(255, 255, 255, 0.6);
          backdrop-filter: blur(5px);
          padding: 7px 10px;
        }
        .window-manager-taskbar-wrapper {
          display: flex;
          flex-direction: row;
          flex-wrap: wrap;
          align-items: stretch;
          height: 100%;
          gap: 10px;
        }
        .window-manager-taskbar-item {
          display: flex;
          flex-direction: row;
          align-items: center;
          height: 36px;
          width: 200px;
          border-radius: 6px;
          cursor: pointer;
          background: rgba(0, 0, 0, 0.1);
          border: 1px solid rgba(0, 0, 0, 0.1);
          backdrop-filter: blur(5px);
          transition: all 150ms ease-in-out;
        }
        .window-manager-taskbar-item:hover {
          background: rgba(255, 255, 255, 0.1);
          border-color: rgba(255, 255, 255, 0.6);
        }
        .window-manager-taskbar-item.is-active {
          background: rgba(255, 255, 255, 0.6);
        }
        .window-manager-taskbar-icon {
          flex-shrink: 0;
        }
        .window-manager-taskbar-name {
          flex-grow: 1;
          font-size: 13px;
          overflow: hidden;
          text-overflow: ellipsis;
          max-height: 0.9rem;
          word-break: break-all;
          white-space: nowrap;
        }
        .window-manager-taskbar-btn {
          flex-shrink: 0;
          margin: 10px;
          appearance: none;
          cursor: pointer;
          display: flex;
          flex-direction: row;
          align-items: center;
          justify-content: center;
          font-size: 11px;
          color: rgba(0, 0, 0, 0.3);
          font-family: sans-serif;
          font-weight: 600;
          text-transform: uppercase;
          background: rgba(204, 204, 204, 1);
          border-radius: 100%;
          width: 18px;
          height: 18px;
          border: none;
          outline: none;
          transition: all 150ms ease-in-out;
        }
        .window-manager-taskbar-btn:hover {
          color: rgba(0, 0, 0, 0.6);
        }
      `);
var _tmpl$4 = /* @__PURE__ */ web.template(`<div><div class="window-manager-taskbar-icon"></div><div class="window-manager-taskbar-name"></div><button class="window-manager-taskbar-btn">X`);
var Taskbar = (props) => {
  const [store, actions] = exports.windowStore();
  const openedWindows = () => Object.keys(store.windows);
  const isFocused = (item) => item?.attrs?.zIndex == 1;
  return [_tmpl$3(), (() => {
    const _el$2 = _tmpl$22(), _el$3 = _el$2.firstChild;
    web.insert(_el$3, web.createComponent(solidJs.For, {
      get each() {
        return openedWindows();
      },
      children: (item, index) => {
        const windowProps = store.windows[item];
        return (() => {
          const _el$5 = _tmpl$4(), _el$6 = _el$5.firstChild, _el$7 = _el$6.nextSibling, _el$8 = _el$7.nextSibling;
          _el$5.$$click = () => {
            if (isFocused(windowProps)) {
              actions.minimizeWindow(windowProps?.attrs?.key, true);
            } else {
              actions.focusWindow(windowProps?.attrs?.key);
            }
          };
          web.insert(_el$6, web.createComponent(WindowIcon, {
            get icon() {
              return windowProps?.attrs?.icon;
            },
            get name() {
              return windowProps?.component;
            }
          }));
          web.insert(_el$7, () => windowProps?.attrs?.state?.title || windowProps?.component || "");
          _el$8.$$click = (e) => {
            e.stopPropagation();
            actions.closeWindow(windowProps?.attrs?.key);
          };
          web.effect(() => web.className(_el$5, `${"window-manager-taskbar-item"} ${isFocused(windowProps) ? "is-active" : ""}`));
          return _el$5;
        })();
      }
    }));
    return _el$2;
  })(), _tmpl$32()];
};
var Taskbar_default = Taskbar;
web.delegateEvents(["click"]);
var _tmpl$5 = /* @__PURE__ */ web.template(`<div><div class="window-controller-resize resize-dir-n"></div><div class="window-controller-resize resize-dir-s"></div><div class="window-controller-resize resize-dir-e"></div><div class="window-controller-resize resize-dir-w"></div><div class="window-controller-resize resize-dir-nw"></div><div class="window-controller-resize resize-dir-ne"></div><div class="window-controller-resize resize-dir-se"></div><div class="window-controller-resize resize-dir-sw"></div><div class="window-controller-container"><div class="window-controller-header"><div class="window-controller-header-icon"></div><div class="window-controller-header-title"><span></span></div><div class="window-controller-header-controls"><button type="button" class="window-controller-header-btn-minimize">_</button><button type="button" class="window-controller-header-btn-maximize">\u2610</button><button type="button" class="window-controller-header-btn-close">X</button></div></div><div class="window-controller-inner"></div></div><style>
        .window-controller-wrapper {
          position: absolute;
          top: 10px;
          left: 10px;
        }
        .window-controller-wrapper.is-animating {
          transition: all 150ms ease-in-out;
        }
        .window-controller-container {
          height: 100%;
          overflow: hidden;
          display: flex;
          flex-direction: column;
          align-items: stretch;
          justify-content: center;
          background: rgba(255, 255, 255, 0.84);
          backdrop-filter: blur(5px);
          border-radius: 8px;
          box-shadow: 1px 1px 15px rgba(0, 0, 0, 0.15);
          border: 0.5px solid #ccc;
        }
        .window-controller-wrapper.is-animating .window-controller-container {
          transition: all 150ms ease-in-out;
        }
        .window-controller-wrapper.is-maximized .window-controller-container {
          background: rgba(255, 255, 255, 1);
          border-radius: 0px;
        }
        .window-controller-header {
          flex-shrink: 0;
          width: 100%;
          display: flex;
          flex-direction: row;
          align-items: center;
          align-content: center;
          justify-content: center;
          background: #f1efff;
          border-bottom: 0.5px solid #ccc;
        }
        .window-controller-inner {
          flex: 1;
          overflow: hidden;
          overflow-y: auto;
        }
        .window-controller-header-icon {
          flex-shrink: 0;
          cursor: default;
        }
        .window-controller-header-title {
          flex: 1;
          cursor: default;
          text-overflow: ellipsis;
          white-space: nowrap;
          overflow: hidden;
        }
        .window-controller-header-controls {
          flex-shrink: 0;
          padding: 10px;
          display: flex;
          flex-direction: row;
        }
        .window-controller-header-controls button {
          appearance: none;
          cursor: pointer;
          display: flex;
          flex-direction: row;
          align-items: center;
          justify-content: center;
          font-size: 11px;
          color: rgba(0, 0, 0, 0.3);
          font-family: sans-serif;
          font-weight: 600;
          text-transform: uppercase;
          background: rgba(204, 204, 204, 1);
          border-radius: 100%;
          width: 18px;
          height: 18px;
          margin-left: 10px;
          border: none;
          outline: none;
          transition: all 150ms ease-in-out;
        }
        .window-controller-header-controls button:hover {
          color: rgba(0, 0, 0, 0.8);
          background: rgba(177, 177, 177, 1);
        }
        .window-controller-header-controls .window-controller-header-btn-close {
          background: rgba(209, 0, 0, 0.3);
        }
        .window-controller-header-controls .window-controller-header-btn-close:hover {
          background: rgba(209, 0, 0, 0.6);
        }
        .window-controller-header-controls .window-controller-header-btn-minimize {
          padding-bottom: 8px;
        }
        .window-controller-loader {
          position: relative;
        }
        .window-controller-loader-bar {
          position: absolute;
          top: 0;
          left: 0;
          width: 0%;
          box-shadow: 1px 0px 1px 1px rgba(255, 0, 0, 0.4);
          transition: all 150ms ease-in-out;
          animation: windowLoaderAnimation 1s infinite;
        }
        @keyframes windowLoaderAnimation {
          from { width: 0%; }
          to { width: 100%; }
        }
        .window-controller-resize {
          position: absolute;
          // box-shadow: 0px 0px 0px 1px red;
          z-index: 10;
        }
        .window-controller-resize.resize-dir-n,
        .window-controller-resize.resize-dir-s {
          left: 0;
          right: 0;
          height: 8px;
        }
        .window-controller-resize.resize-dir-n {
          top: -4px;
          cursor: n-resize;
        }
        .window-controller-resize.resize-dir-s {
          bottom: -4px;
          cursor: s-resize;
        }
        .window-controller-resize.resize-dir-w,
        .window-controller-resize.resize-dir-e {
          top: 0;
          bottom: 0;
          width: 8px;
        }
        .window-controller-resize.resize-dir-w {
          left: -4px;
          cursor: w-resize;
        }
        .window-controller-resize.resize-dir-e {
          right: -4px;
          cursor: e-resize;
        }
        .window-controller-resize.resize-dir-nw,
        .window-controller-resize.resize-dir-ne,
        .window-controller-resize.resize-dir-sw,
        .window-controller-resize.resize-dir-se {
          width: 8px;
          height: 8px;
          // box-shadow: 0px 0px 0px 1px green;
        }
        .window-controller-resize.resize-dir-nw {
          top: -4px;
          left: -4px;
          cursor: nw-resize;
        }
        .window-controller-resize.resize-dir-ne {
          top: -4px;
          right: -4px;
          cursor: ne-resize;
        }
        .window-controller-resize.resize-dir-sw {
          bottom: -4px;
          left: -4px;
          cursor: sw-resize;
        }
        .window-controller-resize.resize-dir-se {
          bottom: -4px;
          right: -4px;
          cursor: se-resize;
        }
      `);
var _tmpl$23 = /* @__PURE__ */ web.template(`<div class="window-controller-loader"><div class="window-controller-loader-bar">`);
var _tmpl$33 = /* @__PURE__ */ web.template(`<div><p>Something went terribly wrong.</p><p>ERROR: `);
function WindowControls(attrs) {
  const [props, rest] = solidJs.splitProps(attrs, ["component", "props", "attrs", "controls", "loadWindow", "windowApi"]);
  const [store, actions] = exports.windowStore();
  const [windowState, setWindowState] = solidJs.createSignal({
    title: "Loading...",
    loading: true
  });
  const [isAnimating, setIsAnimating] = solidJs.createSignal(false);
  solidJs.createEffect(() => {
    let newState = windowState();
    const syncKeys = ["title", "loading"];
    let shouldUpdate = false;
    let newAttrState = {};
    syncKeys.forEach((key) => {
      if (newState?.[key] !== props?.attrs?.state?.[key]) {
        shouldUpdate = true;
        newAttrState[key] = newState[key];
      }
    });
    if (shouldUpdate) {
      actions.updateWindowState(props?.attrs?.key, newAttrState);
    }
  });
  let prevAttrs = {
    ...props.attrs
  };
  let animatingTimeout;
  solidJs.createEffect(() => {
    if (prevAttrs?.maximized !== props.attrs?.maximized || prevAttrs?.minimized !== props.attrs?.minimized) {
      setIsAnimating(true);
      if (animatingTimeout) {
        clearTimeout(animatingTimeout);
        animatingTimeout = null;
      }
      animatingTimeout = setTimeout(() => {
        setIsAnimating(false);
      }, 150);
    }
    prevAttrs = {
      ...props.attrs
    };
  });
  const NewComponent = props?.loadWindow?.(props);
  return (() => {
    const _el$ = _tmpl$5(), _el$2 = _el$.firstChild, _el$3 = _el$2.nextSibling, _el$4 = _el$3.nextSibling, _el$5 = _el$4.nextSibling, _el$6 = _el$5.nextSibling, _el$7 = _el$6.nextSibling, _el$8 = _el$7.nextSibling, _el$9 = _el$8.nextSibling, _el$10 = _el$9.nextSibling, _el$11 = _el$10.firstChild, _el$12 = _el$11.firstChild, _el$13 = _el$12.nextSibling, _el$14 = _el$13.firstChild, _el$15 = _el$13.nextSibling, _el$16 = _el$15.firstChild, _el$17 = _el$16.nextSibling, _el$18 = _el$17.nextSibling, _el$19 = _el$11.nextSibling;
    web.insert(_el$12, web.createComponent(WindowIcon, {
      get icon() {
        return props?.attrs?.icon;
      },
      get name() {
        return props?.component;
      }
    }));
    web.insert(_el$14, () => windowState().title);
    _el$16.$$click = () => props?.controls?.minimize(true);
    _el$17.$$click = () => props?.controls?.maximize(!props.attrs.maximized);
    _el$18.$$click = () => props?.controls?.close();
    web.insert(_el$10, web.createComponent(solidJs.Show, {
      get when() {
        return windowState().loading;
      },
      children: () => _tmpl$23()
    }), _el$19);
    web.insert(_el$19, web.createComponent(solidJs.ErrorBoundary, {
      fallback: (error) => (() => {
        const _el$21 = _tmpl$33(), _el$22 = _el$21.firstChild, _el$23 = _el$22.nextSibling; _el$23.firstChild;
        web.insert(_el$23, () => error.message, null);
        return _el$21;
      })(),
      get children() {
        return web.createComponent(solidJs.Suspense, {
          get fallback() {
            return [];
          },
          get children() {
            return web.createComponent(web.Dynamic, web.mergeProps({
              component: NewComponent
            }, () => props.props, {
              get windowApi() {
                return props.windowApi;
              },
              windowUpdateProps: (i) => {
                setWindowState((v) => ({
                  ...v,
                  loading: false,
                  ...i
                }));
              },
              close: () => props?.controls?.close(),
              minimize: (val = true) => props?.controls?.minimize(val),
              maximize: (val = true) => props?.controls?.maximize(val)
            }));
          }
        });
      }
    }));
    web.effect((_p$) => {
      const _v$ = `window-controller-wrapper ${props.attrs.maximized ? "is-maximized" : ""} ${props.attrs.minimized ? "is-minimized" : ""} ${isAnimating() ? "is-animating" : ""}`, _v$2 = {
        "z-index": 1e4 - props.attrs.zIndex,
        ...props.attrs.maximized ? {
          left: 0,
          top: "60px",
          right: 0,
          bottom: 0,
          width: "100%",
          height: "100%"
        } : {
          left: `${props.attrs.pos?.[0] || 100}px`,
          top: `${props.attrs.pos?.[1] || 100}px`,
          width: `${props.attrs.size?.[0] || 100}px`,
          height: `${props.attrs.size?.[1] || 100}px`
        },
        ...props.attrs.minimized ? {
          transform: `translate(0, ${-(props.attrs.pos?.[1] || 100) - (props.attrs.size?.[1] || 100)}px)`
        } : {
          transform: "translate(0, 0)"
        }
      }, _v$3 = props.attrs.key;
      _v$ !== _p$._v$ && web.className(_el$, _p$._v$ = _v$);
      _p$._v$2 = web.style(_el$, _v$2, _p$._v$2);
      _v$3 !== _p$._v$3 && web.setAttribute(_el$, "window-key", _p$._v$3 = _v$3);
      return _p$;
    }, {
      _v$: void 0,
      _v$2: void 0,
      _v$3: void 0
    });
    return _el$;
  })();
}
web.delegateEvents(["click"]);

// src/WindowManager.tsx
var _tmpl$6 = /* @__PURE__ */ web.template(`<div class="window-manager-wrapper"><style>
        .window-manager-wrapper, .window-manager-wrapper * {
          box-sizing: border-box;
        }
        .window-manager-wrapper {
          width: 100vw;
          height: 100vh;
          overflow: hidden;
          position: relative;
        }
        .window-manager-wrapper::before {
          content:'';
          position: absolute;
          top: 0;
          left: 0;
          right: 0;
          bottom: 0;
          background: url(https://random.imagecdn.app/1920/1080?blur=6) no-repeat;
          background-size: cover;
          background-position: center center;
          z-index: -1;
          opacity: 1;
          filter: blur(4px);
          transform: scale(1.04);
        }
        .window-manager-wrapper::after {
          content:'';
          position: absolute;
          top: 0;
          left: 0;
          right: 0;
          bottom: 0;
          background: black;
          z-index: -1;
          opacity: 0.6;
        }
      `);
function WindowManager(attrs) {
  const child = solidJs.children(() => attrs.children);
  const [props, rest] = solidJs.splitProps(attrs, ["onReady", "loadWindow", "options"]);
  const [store, actions] = exports.windowStore();
  const openedWindows = () => Object.keys(store.windows);
  const openWindow = (component, props2 = {}) => {
    actions.openWindow(component, props2);
  };
  let mainWrapper;
  const state = {
    active: false,
    start: [0, 0],
    windowStart: [0, 0],
    windowSize: [0, 0],
    activeWindowKey: null,
    moveAction: null,
    resizeDir: null,
    offset: [0, 0]
  };
  const events = {
    onMouseDown: (event) => {
      let foundHeader = false;
      let foundResize = false;
      let foundWindow = false;
      let activeWindow = null;
      const path = eventPath(event);
      path.forEach((el) => {
        const className = String(el?.className || "");
        if (className.indexOf("window-controller-header") !== -1) {
          foundHeader = true;
        }
        if (className.indexOf("window-controller-resize") !== -1) {
          foundResize = true;
          if (className.indexOf("resize-dir-n") !== -1)
            foundResize = "n";
          if (className.indexOf("resize-dir-s") !== -1)
            foundResize = "s";
          if (className.indexOf("resize-dir-e") !== -1)
            foundResize = "e";
          if (className.indexOf("resize-dir-w") !== -1)
            foundResize = "w";
          if (className.indexOf("resize-dir-nw") !== -1)
            foundResize = "nw";
          if (className.indexOf("resize-dir-ne") !== -1)
            foundResize = "ne";
          if (className.indexOf("resize-dir-se") !== -1)
            foundResize = "se";
          if (className.indexOf("resize-dir-sw") !== -1)
            foundResize = "sw";
        }
        if ((el?.attributes || {}).hasOwnProperty("window-key")) {
          foundWindow = true;
          state.activeWindowKey = el.attributes?.["window-key"]?.value;
          if (store.windows.hasOwnProperty(state.activeWindowKey)) {
            activeWindow = store.windows[state.activeWindowKey];
            if (activeWindow.attrs.zIndex > 1) {
              actions.focusWindow(state.activeWindowKey);
            }
          }
        }
      });
      console.log("mousedown", {
        foundHeader,
        foundResize,
        foundWindow,
        el: event
      });
      if (!foundWindow || !foundHeader && !foundResize)
        return;
      event.preventDefault();
      event.stopPropagation();
      state.active = true;
      state.start = [event.clientX, event.clientY];
      if (foundHeader)
        state.moveAction = "move";
      if (foundResize != false) {
        state.moveAction = "resize";
        state.resizeDir = foundResize;
      }
      if (activeWindow) {
        state.windowStart = activeWindow.attrs.pos;
        state.windowSize = activeWindow.attrs.size || [100, 100];
        state.offset = [state.start[0] - state.windowStart[0], state.start[1] - state.windowStart[1]];
      }
    },
    onMouseMove: (event) => {
      if (!state.active)
        return;
      event.preventDefault();
      event.stopPropagation();
      if (state.moveAction == "move") {
        actions.moveWindow(state.activeWindowKey, [event.clientX - state.offset[0], event.clientY - state.offset[1]]);
      }
      if (state.moveAction == "resize") {
        let diff = [event.clientX - state.start[0], event.clientY - state.start[1]];
        switch (state.resizeDir) {
          case "n":
            actions.resizeWindow(state.activeWindowKey, (size) => [size?.[0], state.windowSize[1] - diff[1]]);
            actions.moveWindow(state.activeWindowKey, (pos) => [pos?.[0], state.windowStart[1] + diff[1]]);
            break;
          case "ne":
            actions.resizeWindow(state.activeWindowKey, (size) => [state.windowSize[0] + diff[0], state.windowSize[1] - diff[1]]);
            actions.moveWindow(state.activeWindowKey, (pos) => [pos?.[0], state.windowStart[1] + diff[1]]);
            break;
          case "nw":
            actions.resizeWindow(state.activeWindowKey, (size) => [state.windowSize[0] - diff[0], state.windowSize[1] - diff[1]]);
            actions.moveWindow(state.activeWindowKey, (pos) => [state.windowStart[0] + diff[0], state.windowStart[1] + diff[1]]);
            break;
          case "s":
            actions.resizeWindow(state.activeWindowKey, (size) => [size?.[0], state.windowSize[1] + diff[1]]);
            break;
          case "se":
            actions.resizeWindow(state.activeWindowKey, (size) => [state.windowSize[0] + diff[0], state.windowSize[1] + diff[1]]);
            break;
          case "sw":
            actions.resizeWindow(state.activeWindowKey, (size) => [state.windowSize[0] - diff[0], state.windowSize[1] + diff[1]]);
            actions.moveWindow(state.activeWindowKey, (pos) => [state.windowStart[0] + diff[0], pos?.[1]]);
            break;
          case "e":
            actions.resizeWindow(state.activeWindowKey, (size) => [state.windowSize[0] + diff[0], size?.[1]]);
            break;
          case "w":
            actions.resizeWindow(state.activeWindowKey, (size) => [state.windowSize[0] - diff[0], size?.[1]]);
            actions.moveWindow(state.activeWindowKey, (pos) => [state.windowStart[0] + diff[0], pos?.[1]]);
            break;
        }
      }
    },
    onMouseUp: (event) => {
      if (!state.active)
        return;
      event.preventDefault();
      event.stopPropagation();
      state.active = false;
      state.start = [0, 0];
      state.windowStart = [0, 0];
      state.windowSize = [0, 0];
      state.activeWindowKey = null;
      state.moveAction = null;
      state.resizeDir = null;
      state.offset = [0, 0];
    }
  };
  const windowApi = {
    openWindow
  };
  solidJs.onMount(() => {
    props?.onReady?.(windowApi);
  });
  return (() => {
    const _el$ = _tmpl$6(), _el$2 = _el$.firstChild;
    const _ref$ = mainWrapper;
    typeof _ref$ === "function" ? web.use(_ref$, _el$) : mainWrapper = _el$;
    web.spread(_el$, events, false, true);
    web.insert(_el$, web.createComponent(Taskbar_default, {}), _el$2);
    web.insert(_el$, child, _el$2);
    web.insert(_el$, web.createComponent(solidJs.For, {
      get each() {
        return openedWindows();
      },
      children: (item, index) => {
        const windowProps = store.windows[item];
        return web.createComponent(WindowControls, web.mergeProps(windowProps, {
          get loadWindow() {
            return props.loadWindow;
          },
          windowApi,
          controls: {
            close: () => actions.closeWindow(windowProps?.attrs?.key),
            maximize: (maximized) => actions.maximizeWindow(windowProps?.attrs?.key, maximized),
            minimize: (minimized) => actions.minimizeWindow(windowProps?.attrs?.key, minimized)
          }
        }));
      }
    }), _el$2);
    return _el$;
  })();
}

exports.WindowManager = WindowManager;
